
// from http://astronomy.swin.edu.au/~pbourke/other/linearregression/
// with small fixes (added includes, s/abs/fabs/, r^2, ...)
// also see http://en.wikipedia.org/wiki/Simple_linear_regression

#include <iostream>
#include <string>
#include <vector>
#include <math.h>
using namespace std;

// plain C-style linregress (albeit in a C++ file)
int LinRegress(double *x, double *y, int n, 
               double *a, double *b, double *r) {
  int i;
  double sumx=0, sumy=0, sumx2=0, sumy2=0, sumxy=0;
  double sxx, syy, sxy;

  *a = *b = *r = 0;                     // reset just in case
  if (n < 2)
    return(-1);

  for (i=0;i<n;i++) { // compute sums of x, y, xy, x^2, y^2
    sumx += x[i];           sumy += y[i];
    sumx2 += (x[i] * x[i]); sumy2 += (y[i] * y[i]);
    sumxy += (x[i] * y[i]);
  }
  sxx = sumx2 - sumx * sumx / n;        // variance(x)
  syy = sumy2 - sumy * sumy / n;        // variance(y)  
  sxy = sumxy - sumx * sumy / n;        // covariance(x, y)
  
  if (fabs(sxx) == 0)    // Infinite slope (b), non existant intercept (a) 
    return(-1);
  *b = sxy / sxx;        // beta = cov(x,y) / var(x); and alpha = 
  *a = sumy / n - (*b) * sumx / n; // mean(y) - beta * mean(x)
  if (fabs(syy) == 0)    // Compute the correlation coefficient
    *r = 1;
  else
    *r = sxy / sqrt(sxx * syy);  // cov / (stddev(x)*stddev(y))

  return(0);
}


int main()
{
  const int count = 10;
  double times[count] = { 
    -0.20707, 0.706672, 1.63739, 2.03117, 3.31874, 
    5.38201, 6.79971, 6.31814, 8.20829, 8.53994 };
  double water[count] = { 
    -0.319029, 0.0931669, 2.17286, 2.76818, 3.56743, 
    4.11772, 5.52709, 7.46613, 8.7654, 9.58096 };

  double a, b, r;
  int rc;
  rc = LinRegress(times, water, count, &a, &b, &r);
  cout << "Intercept = " << a << endl
       << "Slope     = " << b << endl
       << "R^2       = " << r*r << endl;
}
